@@ -12,7 +12,7 @@ import aiofiles
 import aiohttp
 from tqdm import tqdm
 
-__version__ = "2.0.2"
+__version__ = "2.0.3"
 CONFIG_PATH = os.path.join(
     os.path.dirname(os.path.abspath(__file__)), ".canvassyncer.json"
 )
@@ -29,8 +29,7 @@ class AsyncDownloader:
         async with self.sem:
             async with self.sess.get(src, proxy=self.config.get("proxies")) as res:
                 if res.status != 200:
-                    print(f"{src} Download failed: {res.status}")
-                    return
+                    return self.failures.append(f"{src} => {dst}")
                 async with aiofiles.open(dst, "+wb") as f:
                     while True:
                         chunk = await res.content.read(1024 * 4)
@@ -40,10 +39,16 @@ class AsyncDownloader:
                         self.tqdm.update(len(chunk))
 
     async def start(self, infos, totalSize=0):
-        self.totalSize = totalSize
         self.tqdm = tqdm(total=totalSize, unit="B", unit_scale=True)
-        self.tasks = [self.downloadOne(src, dst) for src, dst in infos]
-        return await asyncio.gather(*self.tasks)
+        self.failures = []
+        await asyncio.gather(
+            *[asyncio.create_task(self.downloadOne(src, dst)) for src, dst in infos]
+        )
+        self.tqdm.close()
+        if self.failures:
+            print(f"Fail to download these {len(self.failures)} file(s):")
+            for text in self.failures:
+                print(text)
 
 
 class CanvasSyncer:
@@ -228,7 +233,7 @@ class CanvasSyncer:
     async def getCourseCodeByCourseID(self):
         await asyncio.gather(
             *[
-                self.getCourseCodeByCourseIDHelper(courseID)
+                asyncio.create_task(self.getCourseCodeByCourseIDHelper(courseID))
                 for courseID in self.config["courseIDs"]
             ]
         )
@@ -341,7 +346,10 @@ class CanvasSyncer:
         print(f"Get {len(self.courseCode)} available courses!")
         print("Finding files on canvas...")
         await asyncio.gather(
-            *[self.getCourseTaskInfo(courseID) for courseID in self.courseCode.keys()]
+            *[
+                asyncio.create_task(self.getCourseTaskInfo(courseID))
+                for courseID in self.courseCode.keys()
+            ]
         )
         if not self.newFiles and not self.laterFiles:
             print("All local files are synced!")
@@ -471,6 +479,11 @@ async def sync():
         if args.debug:
             print(traceback.format_exc())
         exit(1)
+    except aiohttp.ServerDisconnectedError as e:
+        print("ServerDisconnectedError, try to reduce connection count using -c")
+        if args.debug:
+            print(traceback.format_exc())
+        exit(1)
     except KeyboardInterrupt as e:
         raise (e)
     except Exception as e:
